// -*- C++ -*-
/*!
 * @file  GoroneSensorDataConvert.h
 * @brief Sensor Data Json Convert
 * @date  $Date$
 *
 * @author East Japan Accounting Center Co.,Ltd.
 *
 * (c)2017 The University of Aizu
 * This software is released under the New BSD License.
 *
 * $Id$
 */

#ifndef GORONESENSORDATACONVERT_H
#define GORONESENSORDATACONVERT_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>
#include "VersatileOverheadSystem.h"
#include "json.h"
#include "writer.h"
#include <chrono>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="port_stub_h">
// </rtc-template>

using namespace RTC;

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

struct GoroneSensorData_st{
	double_type	dGyro_Roll;
	double_type	dGyro_Pitch;
	double_type	dGyro_Yaw;
	double_type	dSpeed_XAxis;
	double_type	dSpeed_YAxis;
	double_type	dSpeed_Yaw;
	float_type	fCompass;
};

#define GORONE_1		0
#define GORONE_2		1
#define GORONE_3		2
#define GORONE_4		3

/*!
 * @class GoroneSensorDataConvert
 * @brief Sensor Data Json Convert
 *
 * 別紙"機能仕様書_ゴローンセンサデータ変換RTC.docx"参照
 *
 * 別紙"機能仕様書_ゴローンセンサデータ変換RTC.docx"参照
 *
 * 別紙"機能仕様書_ゴローンセンサデータ変換RTC.docx"参照
 *
 * 別紙"機能仕様書_ゴローンセンサデータ変換RTC.docx"参照
 *
 */
class GoroneSensorDataConvert
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  GoroneSensorDataConvert(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~GoroneSensorDataConvert();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   * ゴローンセンサデータ変換RTCの初期化処理
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   * ゴローンセンサデータ変換RTCの終了処理
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   * ゴローンセンサデータ変換RTCのアクティブ処理
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   * ゴローンセンサデータ変換RTCの非アクティブ処理
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   * ゴローンセンサデータ変換RTCの周期処理
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   * ゴローンセンサデータ変換RTCのエラー遷移処理
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * ゴローンセンサデータ出力周期を調整する
   * - Name: Output_Cycle Output_Cycle
   * - DefaultValue: 0
   * - Unit: ms
   * - Range: 0<=x<=10000
   * - Constraint: -
   */
  long int m_Output_Cycle;

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  RTC::TimedOrientation3D m_inGorone1GyroData;
  /*!
   * ゴローン1のGyroデータを取得するデータポート
   * - Type: RTC::TimedOrientation3D
   * - Number: 1
   */
  InPort<RTC::TimedOrientation3D> m_inGorone1GyroDataIn;
  RTC::TimedVelocity2D m_inGorone1SpeedData;
  /*!
   * ゴローン1のSpeedデータを取得するデータポート
   * - Type: RTC::TimedVelocity2D
   * - Number: 1
   */
  InPort<RTC::TimedVelocity2D> m_inGorone1SpeedDataIn;
  RTC::TimedFloat m_inGorone1CompassData;
  /*!
   * ゴローン1のコンパスデータを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inGorone1CompassDataIn;
  RTC::TimedOrientation3D m_inGorone2GyroData;
  /*!
   * ゴローン2のGyroデータを取得するデータポート
   * - Type: RTC::TimedOrientation3D
   * - Number: 1
   */
  InPort<RTC::TimedOrientation3D> m_inGorone2GyroDataIn;
  RTC::TimedVelocity2D m_inGorone2SpeedData;
  /*!
   * ゴローン2のSpeedデータを取得するデータポート
   * - Type: RTC::TimedVelocity2D
   * - Number: 1
   */
  InPort<RTC::TimedVelocity2D> m_inGorone2SpeedDataIn;
  RTC::TimedFloat m_inGorone2CompassData;
  /*!
   * ゴローン2のコンパスデータを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inGorone2CompassDataIn;
  RTC::TimedOrientation3D m_inGorone3GyroData;
  /*!
   * ゴローン3のGyroデータを取得するデータポート
   * - Type: RTC::TimedOrientation3D
   * - Number: 1
   */
  InPort<RTC::TimedOrientation3D> m_inGorone3GyroDataIn;
  RTC::TimedVelocity2D m_inGorone3SpeedData;
  /*!
   * ゴローン3のSpeedデータを取得するデータポート
   * - Type: RTC::TimedVelocity2D
   * - Number: 1
   */
  InPort<RTC::TimedVelocity2D> m_inGorone3SpeedDataIn;
  RTC::TimedFloat m_inGorone3CompassData;
  /*!
   * ゴローン3のコンパスデータを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inGorone3CompassDataIn;
  RTC::TimedOrientation3D m_inGorone4GyroData;
  /*!
   * ゴローン4のGyroデータを取得するデータポート
   * - Type: RTC::TimedOrientation3D
   * - Number: 1
   */
  InPort<RTC::TimedOrientation3D> m_inGorone4GyroDataIn;
  RTC::TimedVelocity2D m_inGorone4SpeedData;
  /*!
   * ゴローン4のSpeedデータを取得するデータポート
   * - Type: RTC::TimedVelocity2D
   * - Number: 1
   */
  InPort<RTC::TimedVelocity2D> m_inGorone4SpeedDataIn;
  RTC::TimedFloat m_inGorone4CompassData;
  /*!
   * ゴローン4のコンパスデータを取得するデータポート
   * - Type: RTC::TimedFloat
   * - Number: 1
   */
  InPort<RTC::TimedFloat> m_inGorone4CompassDataIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  RTC::TimedStringSeq m_outGoroneSensorData;
  /*!
   * ゴローンのセンサデータをJson形式で台数分提供するデータポート
   * - Type: RTC::TimedStringSeq
   * - Number: 4
   * - Frequency: 500ms周期
   * - Operation Cycle: 500ms周期
   */
  OutPort<RTC::TimedStringSeq> m_outGoroneSensorDataOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

	 void nvdGoroneDataRamInit(void);
	 bool_type blGoroneDataPortRead(void);
	 uhword_type uhwGoroneSensorDataOutput(void);

	 InPort<RTC::TimedOrientation3D> *m_inGyroDataIn_p[GORONE_CONNECT_MAX];
	 InPort<RTC::TimedVelocity2D> *m_inSpeedDataIn_p[GORONE_CONNECT_MAX];
	 InPort<RTC::TimedFloat> *m_inCompassDataIn_p[GORONE_CONNECT_MAX];
	 RTC::TimedOrientation3D *m_inGyroData_p[GORONE_CONNECT_MAX];
	 RTC::TimedVelocity2D *m_inSpeedData_p[GORONE_CONNECT_MAX];
	 RTC::TimedFloat *m_inCompassData_p[GORONE_CONNECT_MAX];
	 GoroneSensorData_st m_stGoroneData[GORONE_CONNECT_MAX];

	 std::chrono::system_clock::time_point  m_LastTime;	//前回のセンサデータ出力時間を保持
};


extern "C"
{
  DLL_EXPORT void GoroneSensorDataConvertInit(RTC::Manager* manager);
};

#endif // GORONESENSORDATACONVERT_H
