// -*- C++ -*-
/*!
 * @file  GoroneSensorDataConvert.cpp
 * @brief Sensor Data Json Convert
 * @date $Date$
 *
 * @author East Japan Accounting Center Co.,Ltd.
 *
 * (c)2017 The University of Aizu
 * This software is released under the New BSD License.
 *
 * $Id$
 */

#include "GoroneSensorDataConvert.h"

// Module specification
// <rtc-template block="module_spec">
static const char* goronesensordataconvert_spec[] =
  {
    "implementation_id", "GoroneSensorDataConvert",
    "type_name",         "GoroneSensorDataConvert",
    "description",       "Sensor Data Json Convert",
    "version",           "0.1",
    "vendor",            "EAC",
    "category",          "Sensor",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.Output_Cycle", "0",
    // Widget
    "conf.__widget__.Output_Cycle", "spin.500",
    // Constraints
    "conf.__constraints__.Output_Cycle", "0<=x<=10000",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
GoroneSensorDataConvert::GoroneSensorDataConvert(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_inGorone1GyroDataIn("inGorone1GyroData", m_inGorone1GyroData),
    m_inGorone1SpeedDataIn("inGorone1SpeedData", m_inGorone1SpeedData),
    m_inGorone1CompassDataIn("inGorone1CompassData", m_inGorone1CompassData),
    m_inGorone2GyroDataIn("inGorone2GyroData", m_inGorone2GyroData),
    m_inGorone2SpeedDataIn("inGorone2SpeedData", m_inGorone2SpeedData),
    m_inGorone2CompassDataIn("inGorone2CompassData", m_inGorone2CompassData),
    m_inGorone3GyroDataIn("inGorone3GyroData", m_inGorone3GyroData),
    m_inGorone3SpeedDataIn("inGorone3SpeedData", m_inGorone3SpeedData),
    m_inGorone3CompassDataIn("inGorone3CompassData", m_inGorone3CompassData),
    m_inGorone4GyroDataIn("inGorone4GyroData", m_inGorone4GyroData),
    m_inGorone4SpeedDataIn("inGorone4SpeedData", m_inGorone4SpeedData),
    m_inGorone4CompassDataIn("inGorone4CompassData", m_inGorone4CompassData),
    m_outGoroneSensorDataOut("outGoroneSensorData", m_outGoroneSensorData)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
GoroneSensorDataConvert::~GoroneSensorDataConvert()
{
}


/*!
 * ゴローンセンサデータ変換RTCの初期化処理
 */
RTC::ReturnCode_t GoroneSensorDataConvert::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("inGorone1GyroData", m_inGorone1GyroDataIn);
  addInPort("inGorone1SpeedData", m_inGorone1SpeedDataIn);
  addInPort("inGorone1CompassData", m_inGorone1CompassDataIn);
  addInPort("inGorone2GyroData", m_inGorone2GyroDataIn);
  addInPort("inGorone2SpeedData", m_inGorone2SpeedDataIn);
  addInPort("inGorone2CompassData", m_inGorone2CompassDataIn);
  addInPort("inGorone3GyroData", m_inGorone3GyroDataIn);
  addInPort("inGorone3SpeedData", m_inGorone3SpeedDataIn);
  addInPort("inGorone3CompassData", m_inGorone3CompassDataIn);
  addInPort("inGorone4GyroData", m_inGorone4GyroDataIn);
  addInPort("inGorone4SpeedData", m_inGorone4SpeedDataIn);
  addInPort("inGorone4CompassData", m_inGorone4CompassDataIn);
  
  // Set OutPort buffer
  addOutPort("outGoroneSensorData", m_outGoroneSensorDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("Output_Cycle", m_Output_Cycle, "0");
  // </rtc-template>

#ifdef _DEBUG
  std::cout << "[info]GoroneSensorDataConvert::onInitialize()" << std::endl;
#endif

  nvdGoroneDataRamInit();
  m_outGoroneSensorData.data.length(GORONE_CONNECT_MAX);

  m_inGyroDataIn_p[GORONE_1] = &m_inGorone1GyroDataIn;
  m_inGyroDataIn_p[GORONE_2] = &m_inGorone2GyroDataIn;
  m_inGyroDataIn_p[GORONE_3] = &m_inGorone3GyroDataIn;
  m_inGyroDataIn_p[GORONE_4] = &m_inGorone4GyroDataIn;
  m_inGyroData_p[GORONE_1] = &m_inGorone1GyroData;
  m_inGyroData_p[GORONE_2] = &m_inGorone2GyroData;
  m_inGyroData_p[GORONE_3] = &m_inGorone3GyroData;
  m_inGyroData_p[GORONE_4] = &m_inGorone4GyroData;

  m_inSpeedDataIn_p[GORONE_1] = &m_inGorone1SpeedDataIn;
  m_inSpeedDataIn_p[GORONE_2] = &m_inGorone2SpeedDataIn;
  m_inSpeedDataIn_p[GORONE_3] = &m_inGorone3SpeedDataIn;
  m_inSpeedDataIn_p[GORONE_4] = &m_inGorone4SpeedDataIn;
  m_inSpeedData_p[GORONE_1] = &m_inGorone1SpeedData;
  m_inSpeedData_p[GORONE_2] = &m_inGorone2SpeedData;
  m_inSpeedData_p[GORONE_3] = &m_inGorone3SpeedData;
  m_inSpeedData_p[GORONE_4] = &m_inGorone4SpeedData;

  m_inCompassDataIn_p[GORONE_1] = &m_inGorone1CompassDataIn;
  m_inCompassDataIn_p[GORONE_2] = &m_inGorone2CompassDataIn;
  m_inCompassDataIn_p[GORONE_3] = &m_inGorone3CompassDataIn;
  m_inCompassDataIn_p[GORONE_4] = &m_inGorone4CompassDataIn;
  m_inCompassData_p[GORONE_1] = &m_inGorone1CompassData;
  m_inCompassData_p[GORONE_2] = &m_inGorone2CompassData;
  m_inCompassData_p[GORONE_3] = &m_inGorone3CompassData;
  m_inCompassData_p[GORONE_4] = &m_inGorone4CompassData;

  return RTC::RTC_OK;
}

/*!
 * ゴローンセンサデータ変換RTCの終了処理
 */

RTC::ReturnCode_t GoroneSensorDataConvert::onFinalize()
{
#ifdef _DEBUG
	std::cout << "[info]GoroneSensorDataConvert::onFinalize()" << std::endl;
#endif

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t GoroneSensorDataConvert::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GoroneSensorDataConvert::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * ゴローンセンサデータ変換RTCのアクティブ処理
 */

RTC::ReturnCode_t GoroneSensorDataConvert::onActivated(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[info]GoroneSensorDataConvert::onActivated()" << std::endl;
#endif

	//入力データポートの初期化
	while(m_inGorone1GyroDataIn.isEmpty() == FALSE){
		m_inGorone1GyroDataIn.read();
	}
	while(m_inGorone1SpeedDataIn.isEmpty() == FALSE){
		m_inGorone1SpeedDataIn.read();
	}
	while(m_inGorone1CompassDataIn.isEmpty() == FALSE){
		m_inGorone1CompassDataIn.read();
	}
	while(m_inGorone2GyroDataIn.isEmpty() == FALSE){
		m_inGorone2GyroDataIn.read();
	}
	while(m_inGorone2SpeedDataIn.isEmpty() == FALSE){
		m_inGorone2SpeedDataIn.read();
	}
	while(m_inGorone2CompassDataIn.isEmpty() == FALSE){
		m_inGorone2CompassDataIn.read();
	}
	while(m_inGorone3GyroDataIn.isEmpty() == FALSE){
		m_inGorone3GyroDataIn.read();
	}
	while(m_inGorone3SpeedDataIn.isEmpty() == FALSE){
		m_inGorone3SpeedDataIn.read();
	}
	while(m_inGorone3CompassDataIn.isEmpty() == FALSE){
		m_inGorone3CompassDataIn.read();
	}
	while(m_inGorone4GyroDataIn.isEmpty() == FALSE){
		m_inGorone4GyroDataIn.read();
	}
	while(m_inGorone4SpeedDataIn.isEmpty() == FALSE){
		m_inGorone4SpeedDataIn.read();
	}
	while(m_inGorone4CompassDataIn.isEmpty() == FALSE){
		m_inGorone4CompassDataIn.read();
	}

	//経過時間保持変数を初期化
	m_LastTime = std::chrono::system_clock::now();

	return RTC::RTC_OK;
}

/*!
 * ゴローンセンサデータ変換RTCの非アクティブ処理
 */

RTC::ReturnCode_t GoroneSensorDataConvert::onDeactivated(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[info]GoroneSensorDataConvert::onDeactivated()" << std::endl;
#endif

	nvdGoroneDataRamInit();

	return RTC::RTC_OK;
}

/*!
 * ゴローンセンサデータ変換RTCの周期処理
 */

RTC::ReturnCode_t GoroneSensorDataConvert::onExecute(RTC::UniqueId ec_id)
{
	bool_type blRet = FALSE;
	std::chrono::system_clock::time_point  CurrentTime;
	double dElapsedTime = 0;

	CurrentTime = std::chrono::system_clock::now();	//現在の時間を取得
	dElapsedTime = std::chrono::duration_cast<std::chrono::milliseconds>(CurrentTime - m_LastTime).count();	//経過時間をミリ秒に変換

	//コンフィギュレーション出力周期経過後、GPSデータ出力
	if(dElapsedTime >= m_Output_Cycle){
		blRet = blGoroneDataPortRead();
		if(blRet){
			//ゴローンセンサデータ送信
			uhwGoroneSensorDataOutput();
		}
		m_LastTime = CurrentTime;
	}

	return RTC::RTC_OK;
}

/*!
 * ゴローンセンサデータ変換RTCのエラー遷移処理
 */

RTC::ReturnCode_t GoroneSensorDataConvert::onAborting(RTC::UniqueId ec_id)
{
#ifdef _DEBUG
	std::cout << "[info]GoroneSensorDataConvert::onAborting()" << std::endl;
#endif

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t GoroneSensorDataConvert::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GoroneSensorDataConvert::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GoroneSensorDataConvert::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t GoroneSensorDataConvert::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*****************************************************************************************
*	Function Name		：blGoroneDataPortRead
*	Description			：全てのゴローンデータポート読み込み
*	Argument			：なし
*	Return				：TRUE：更新あり、FALSE：更新なし
******************************************************************************************/
bool_type GoroneSensorDataConvert::blGoroneDataPortRead(void)
{
	bool_type blRet = FALSE;
	uhword_type uhwLoop = GORONE_1;

	for(uhwLoop = GORONE_1; uhwLoop < GORONE_CONNECT_MAX; uhwLoop++){
		if(m_inGyroDataIn_p[uhwLoop]->isNew()){
			m_inGyroDataIn_p[uhwLoop]->read();
			m_stGoroneData[uhwLoop].dGyro_Roll = m_inGyroData_p[uhwLoop]->data.r;
			m_stGoroneData[uhwLoop].dGyro_Pitch = m_inGyroData_p[uhwLoop]->data.p;
			m_stGoroneData[uhwLoop].dGyro_Yaw = m_inGyroData_p[uhwLoop]->data.y;
			blRet = TRUE;
		}

		if(m_inSpeedDataIn_p[uhwLoop]->isNew()){
			m_inSpeedDataIn_p[uhwLoop]->read();
			m_stGoroneData[uhwLoop].dSpeed_XAxis = m_inSpeedData_p[uhwLoop]->data.vx;
			m_stGoroneData[uhwLoop].dSpeed_YAxis = m_inSpeedData_p[uhwLoop]->data.vy;
			m_stGoroneData[uhwLoop].dSpeed_Yaw = m_inSpeedData_p[uhwLoop]->data.va;
			blRet = TRUE;
		}

		if(m_inCompassDataIn_p[uhwLoop]->isNew()){
			m_inCompassDataIn_p[uhwLoop]->read();
			m_stGoroneData[uhwLoop].fCompass = m_inCompassData_p[uhwLoop]->data;
			blRet = TRUE;
		}
	}

	return 	blRet;
}


/*****************************************************************************************
*	Function Name		：uhwGoroneSensorDataOutput
*	Description			：ゴローンセンサデータ出力
*	Argument			：なし
*	Return				：正常終了
******************************************************************************************/
uhword_type GoroneSensorDataConvert::uhwGoroneSensorDataOutput(void)
{
	Json::StyledWriter	writer;
	std::string strJson = "";
	uhword_type uhwLoop = GORONE_1;
	Json::Value JsonData;

	for(uhwLoop = GORONE_1; uhwLoop < GORONE_CONNECT_MAX; uhwLoop++){
		JsonData[JSONKEY_SENSOR_GYRO][JSONKEY_ROLL] = m_stGoroneData[uhwLoop].dGyro_Roll;
		JsonData[JSONKEY_SENSOR_GYRO][JSONKEY_PITCH] = m_stGoroneData[uhwLoop].dGyro_Pitch;
		JsonData[JSONKEY_SENSOR_GYRO][JSONKEY_YAW] = m_stGoroneData[uhwLoop].dGyro_Yaw;
		JsonData[JSONKEY_SENSOR_SPEED][JSONKEY_X_AXIS] = m_stGoroneData[uhwLoop].dSpeed_XAxis;
		JsonData[JSONKEY_SENSOR_SPEED][JSONKEY_Y_AXIS] = m_stGoroneData[uhwLoop].dSpeed_YAxis;
		JsonData[JSONKEY_SENSOR_SPEED][JSONKEY_YAW] = m_stGoroneData[uhwLoop].dSpeed_Yaw;
		JsonData[JSONKEY_SENSOR_COMPASS] = m_stGoroneData[uhwLoop].fCompass;

		//jsonフォーマットに変換し出力
		strJson = writer.write(JsonData);
		m_outGoroneSensorData.data[uhwLoop] = strJson.c_str();
		std::cout << "[debug]Gorone No = " << uhwLoop << ",Output = " << strJson << std::endl;
	}
	m_outGoroneSensorDataOut.write();

	return RESULT_SUCCESS;
}

/*****************************************************************************************
*	Function Name		：nvdGoroneDataRamInit
*	Description			：メンバ変数初期化
*	Argument			：なし
*	Return				：なし
******************************************************************************************/
void GoroneSensorDataConvert::nvdGoroneDataRamInit(void)
{
	memset(m_stGoroneData, 0, sizeof(m_stGoroneData));
}

extern "C"
{
 
  void GoroneSensorDataConvertInit(RTC::Manager* manager)
  {
    coil::Properties profile(goronesensordataconvert_spec);
    manager->registerFactory(profile,
                             RTC::Create<GoroneSensorDataConvert>,
                             RTC::Delete<GoroneSensorDataConvert>);
  }
  
};


